import string
from numpy import zeros, arange, prod, ndim, array, float64, flipud, fix
import datetime
#import dbg

def read_infofile(file):
    """reading info indendation file"""
    file_obj = open(file,'r')
    d = 2
    #line 1
    l1 = string.split(file_obj.readline())
    Nd = string.split(l1[1],'x')
    N = zeros(d)
    for i in arange(d):N[i]=float(Nd[i])
    #line 2
    l2 = string.split(file_obj.readline())
    #line 3,4
    h = zeros(2)
    for i in arange(d):
        l = string.split(file_obj.readline())
        hp = string.split(l[1],'=')
        h[i] = float(hp[1])
        if l[2] == 'um':
            h[i] = h[i]*1e-6
    Y = N*h
    file_obj.close()
    return d,N,h,Y

def read_data(file,N):
    """reading data indendation file"""
    file_obj = open(file,'r')
    #line 1
    for i in arange(4):
        line = file_obj.readline()
    list = string.split(line)
    nazev = list[0]
    # preproc
    E = zeros(prod(N))
    line = file_obj.readline()
    list = string.split(line)
    E[0] = float(list[5])
    # radek 6    
    line = file_obj.readline()
    list = string.split(line)
    Eunit = list[0]
    E[1] = float(list[4])
    #ostatni radky
    for i in arange(2,prod(N)):
        line = file_obj.readline()
        list = string.split(line)
        E[i] = float(list[3])
    E = E.reshape(N[0],N[1],order='C').copy()
    return E,Eunit

def io_write(input,file,fmt,delimiter):
    """writing input file
    
    Parameters:
    ----------
    input : dictionary
        variables to save
    file : string
        name of reading file
    fmt : string
        format for writing a data ('%.18e')
    delimiter : string
        string delimiting the numbers
    """
    file_obj = open(file,'w')
    file_obj.write('#INPUT file created on ' + str(datetime.datetime.now()) + '\n')
    for var in input.keys():
        file_obj.write('\n' + var + '\n')
        if ndim(input[var])==0:
            file_obj.write(fmt % input[var] + '\n')
        elif ndim(input[var])==1:
            ncol = input[var].shape[0]
            fmt_ncol = [fmt,]*ncol
            fmt_ncol = delimiter.join(fmt_ncol)
            file_obj.write(fmt_ncol % tuple(input[var]) + '\n')
        elif ndim(input[var])==2:
            ncol = input[var].shape[1]
            fmt_ncol = [fmt,]*ncol
            fmt_ncol = delimiter.join(fmt_ncol)
            for row in input[var]:
                file_obj.write(fmt_ncol % tuple(row) + '\n')      
    file_obj.close()
    return

def io_read(file,variables):
    """reading input file
    
    Parameters:
    ----------
    file : string
        name of reading file
    variables : list of strings
        list of variables to read
    """
    def output_add(output,variable,variable_data,writeable):
        #adding variable to dictionary output
        try:
            if writeable:
                output[variable] = array(variable_data,float64).squeeze()
        except Exception as err:
            print(err)
        return output
    comments = ['#','%']
    variable = None
    output = dict()
    variable_data = None
    writeable = False
    for line in open(file,'r'):
        if line.strip() is None or line.strip()=='' or line[0] in comments:
            continue
        elif line.strip() in variables:
            output_add(output,variable,variable_data,writeable)
            writeable = True
            variable = line.strip()
            variable_data = []
        else:
            line_split = line.split()
            try:
                float(line_split[0])
            except:
                output_add(output,variable,variable_data,writeable)
                writeable = False
                variable_data = []
                continue
            variable_data.append(line_split)
    else:
        output_add(output,variable,variable_data,writeable)
    return output

def read_results(file):
    X = []
    Y = []
    E = []
    nu = []
    ind_no = 0
    variables = ['X','Y','EIT=',"Poisson's",'Indentation']
    for line in open(file,'r'):
        line_split = line.strip().split()
    #    print(line_split)
        if len(line_split) == 0:
            continue
        elif line_split[0] in variables:
            if line_split[0] == 'Indentation' and len(line_split)>2:
                ind_no = int(line_split[2].replace(',','.'))
                continue
            elif line_split[0] == 'EIT=':
                E.append(line_split[1].replace(',','.'))
                continue
            elif line_split[0] == "Poisson's":
                nu.append(line_split[2].replace(',','.'))
                continue
            elif line_split[0] == 'X':
                X.append(line_split[2].strip(':').replace(',','.'))
                continue
            elif line_split[0] == 'Y':
                Y.append(line_split[2].strip(':').replace(',','.'))        
    
    inputs = dict()
    x = array(X,float64)
    y = array(Y,float64)
    x0 = x - x.min()
    y0 = y - y.min()
    diff = zeros(2)
    diff[0] = x.max()-x.min()
    diff[1] = y.max()-y.min()
    
    def factorization(N):
        #returns prime factors of integer N
        n = N
        primes = []
        while n > 1:
            for i in arange(2,fix(n**0.5)+1):
                div = n/i
                if div-fix(div) == 0:
                    primes.append(i)
                    n = div
                    break
            else:
                primes.append(n)
                n=1
        return primes
    
    def get_sizes(primes):
        #returns all possible combination of discretization points in each direction
        rows = []
        sizes = []
        for i in arange(2**len(primes)):
            bi = bin(i)[2:]
            row = 1
            for j in arange(len(bi)):
                if int(bi[-j-1]) != 0:
                    row = row*primes[j]
            if row not in rows:
                rows.append(row)
                sizes.append([row,ind_no/row])
        sizes.sort()
        return sizes
    
    primes = factorization(ind_no)
    sizes = get_sizes(primes)
        
    def is_monotone(a):
        #returns the boolean value about monotonicity of vector a
        if a[0] > a[-1]:
            a = flipud(a)
        result = True
        for i in arange(a.size-1):
            if a[i]>a[i+1]:
                result = False
        return result
    
    sized = []
    components = [] 
    for size in sizes:
        x_rs = x0.reshape(size[0],size[1],order='F')
        y_rs = y0.reshape(size[0],size[1],order='F')
        if size[0]>size[1]:
            x_rs = x_rs.transpose()
            y_rs = y_rs.transpose()
        if is_monotone(x_rs[0]):
            components.append(x_rs)
            components.append(y_rs)
        elif is_monotone(y_rs[0]):
            components.append(y_rs)
            components.append(x_rs)
        else:
            continue
        sized_ok = True
        for j in arange(components[0].shape[0]):
            if not is_monotone(components[0][j]):
                sized_ok = False
                continue
        if sized_ok:
            for j in arange(components[1].shape[1]):
                if not is_monotone(components[1][:,j]):
                    sized_ok = False
                    continue
        if sized_ok: sized.append(size)
    
    N = array(sized[0])
    x0 = x0.reshape(sized[0][0],sized[0][1],order='F')
    y0 = y0.reshape(sized[0][0],sized[0][1],order='F')
    E = array(E)
    nu = array(nu)
    h = diff/(N-1)
    Y = h*N
    E = E.reshape(sized[0][0],sized[0][1],order='F')
    nu = nu.reshape(sized[0][0],sized[0][1],order='F')
    inputs['E'] = E
    inputs['nu'] = nu
    inputs['N'] = N
    inputs['Y'] = Y 
    inputs['h'] = h
    inputs['x'] = x
    inputs['y'] = y
    inputs['x0'] = x0
    inputs['y0'] = y0
    inputs['sizes'] = sizes
    inputs['sized'] = array(sized)
    return inputs
    
