from enthought.traits.api import Str, Float, Int, Enum, File, Array, HasTraits, Event, Button, Instance, Bool, List, Dict
from enthought.traits.ui.api import View, ArrayEditor, CheckListEditor, Item, Group, ButtonEditor
from enthought.traits.ui.menu import NoButtons, OKButton, CancelButton
import io_rw, FFTH_elas
#import dbg
import string
import numpy as np
import matplotlib.pyplot as plt

class EditField(HasTraits):
    field = Array(np.float,editor = ArrayEditor(width = 60))
    def __init__(self,x):
        self.field = x
    
class ControlPanel(HasTraits):
    #panel consisting of Indendation, FFTH Calculation, Postprocessing
    inputs = Dict()
    outputs = Dict()
    field = Instance(EditField)
    #indendation:
    ind_file = File('K1-1#1.results.TXT')
    ind_update = Button()
    ind_E = Array(np.float,editor = ArrayEditor(width = 140))
    ind_nu = Array(np.float,editor = ArrayEditor(width = 140))
    ind_N = Array(np.float, (2,),editor = ArrayEditor(width = 140))
    ind_Y = Array(np.float, (2,),editor = ArrayEditor(width = 140))
    ind_h = Array(np.float, (2,),editor = ArrayEditor(width = 140))
    ind_input = Bool()
    ind_input_name = File('input.txt')
    ind_draw_E = Button()
    ind_draw_nu = Button()
    ind_edit_E = Button()
    ind_edit_nu = Button()
    ind_calc = Bool()
    ind_postpro = Bool()
    ind_make_input = Button()
    #indendation 2: 
    ind2_info_file = File()
    ind2_data_file = File()
    ind2_input_file = Str('input.txt')
    ind2_nu = Float()
    #calculation
    calc_file = File('input.txt')
    calc_output_file  = Str('output.txt')
    tol = Float(1e-4)
    maxit = Int(1e3)
    calc_postpro = Bool()    
    calc = Button()
    #postprocessing
    postproc_file = File('output.txt')
    postproc_update = Button()
    L_homogenized = Array(np.float, (3,3),editor = ArrayEditor(width = 150))
    kit = Array(np.int, (1,3),editor = ArrayEditor(width = 150))
    resnorm = Array(np.float, (1,3),editor = ArrayEditor(width = 150))
    
    def _ind_edit_E_fired(self):
        field2 = EditField(self.ind_E)
        field2.configure_traits()
        print(field2)
        self.ind_E = field2.field
    
    def _ind_update_fired(self):
        input = io_rw.read_results(self.ind_file)
        self.ind_N = input['N']
        self.ind_Y = input['Y']
        self.ind_E = input['E']
        self.ind_nu = input['nu']
        self.ind_h = input['h']
    
#    def _ind_Y_changed(self):
#        self.ind_h = self.ind_Y/self.ind_N
    
    def _ind_h_changed(self):
        self.ind_Y = self.ind_N*self.ind_h
        
    def _ind_draw_E_fired(self):
        from mpl_toolkits.mplot3d import Axes3D
        from matplotlib import cm
        from matplotlib.ticker import LinearLocator, FixedLocator, FormatStrFormatter
        import matplotlib.pyplot as plt
        
        fig = plt.figure()
        ax = Axes3D(fig)
        #ax = fig.gca(projection='3d')
        X = np.arange(0, self.ind_Y[0]-self.ind_h[0], self.ind_N[0])
        Y = np.arange(0, self.ind_Y[1]-self.ind_h[1], self.ind_N[1])
        print(self.ind_N)
        print(self.ind_N[0])
        print(self.ind_N[1])
        print(np.shape(X))
        print(np.shape(Y))
        
        X, Y = np.meshgrid(X, Y)
        Z = self.ind_E
        print(np.shape(Z))
        surf = ax.plot_surface(X, Y, Z, rstride=1, cstride=1, cmap=cm.jet,
                linewidth=0, antialiased=False)
        ax.set_zlim3d(-1.01, 1.01)
        ax.w_zaxis.set_major_locator(LinearLocator(10))
        ax.w_zaxis.set_major_formatter(FormatStrFormatter('%.03f'))
        fig.colorbar(surf, shrink=0.5, aspect=5)
        plt.show()
    
    def _ind_make_input_fired(self):
        try:
            input = dict()
            input['Y'] = self.ind_Y
            input['E'] = self.ind_E
            input['nu'] = self.ind_nu
            fmt = '%.6e';delimiter = ' '
            io_rw.io_write(input,self.ind_input_name,fmt,delimiter)
            self.calc_file = self.ind_input_name
        except Exception as err:
            print(err)

    def _ind2_make_input_fired(self):
        try:
            d,N,h,Y = io_rw.read_infofile(self.ind2_info_file)
            E,Eunit = io_rw.read_data(self.ind2_data_file,N)
            input = dict(E=E,Y=Y,nu=self.ind2_nu)
            fmt = '%.10e';delimiter = ' '
            io_rw.io_write(input,self.ind2_input_file,fmt,delimiter)
        except Exception as err:
            print(err)
    
    def _calc_fired(self):
        variables = ['E','nu','Y','tol','maxit']
        input = io_rw.io_read(self.calc_file,variables)
        K = input['E']/(3*(1-2*input['nu']))
        G = input['E']/(2*(1+input['nu']))
        input['L'] = FFTH_elas.get_L(K,G)
        input['tol'] = self.tol; input['maxit'] = self.maxit
        input['K0'],input['G0'] = FFTH_elas.get_K0_G0(K,G)
        fmt = '%.5e'; delimiter = ' '
        output = FFTH_elas.homogenization(input)
        io_rw.io_write(output, self.calc_output_file, fmt, delimiter)
        self.postproc_file = self.calc_output_file
    
    def _postproc_update_fired(self):
        variables = ['L_homogenized','kit','resnorm']
        output = io_rw.io_read(self.postproc_file,variables)
        self.L_homogenized = output['L_homogenized']
        self.kit = [output['kit']]
        self.resnorm = [output['resnorm']]
        
    view = View(
        Group(Group(Item('ind_file',label="Info file"),
                      Item('ind_update',label="Update",show_label=False),
                      label="Input data from nanoindentation",show_border=True),
            Group(Item('ind_N',label="No. of discretization points",style='readonly'),
                            Item('ind_Y',label="Size of PUC",style='readonly'),
                            Item('ind_h',label="Discretization step"),
                            label="Data from input file",show_border=True),
              Group(Item('ind_input_name',label="Name of input file"),
                    Item('ind_make_input',label="Make input file, (Calculate, Postprocess)",show_label=False),
                            label="Creation of input file",show_border=True,),
                    label="Indentation"),
#        Group(Group(Item('ind2_info_file',label="Info file"),
#                      Item('ind2_data_file',label="Data file",),
#                      Item('ind2_nu',label="Poisson\'s ratio"),
#                      label="Input data from nanoindentation",show_border=True),
#                      Group(Item('ind_input',label="Make input file"),
#                            Item('ind_input_name',label="Name of input file"),
#                            Item('ind_calc',label="Calculate"),
#                            Item('ind_postpro',label="Postprocess"),
#                            Item('ind_make_input',label="Make input file",show_label=False),
#                            label="Creation of input file",show_border=True),
#                    label="Indentation 2"),
        Group(Group(Item('calc_file',label="Input file"),
                      Item('tol',label="Tolerance for convergence"),
                      Item('maxit',label="Maximal no. of iterations"),
                      label="Inputs for FFTH homogenization",show_border=True),
            Group(Item('calc_output_file',label="calc_output_file"),
                    Item('calc',label="Calculate, (Postprocess)",show_label=False),
                      label="Calculation",show_border=True),
                    label="FFT Homogenization"),
        Group(Group(Item('postproc_file'),
                    Item('postproc_update',label="update",show_label=False),
              label="Postprocessing file",show_border=True),
              Group(Item('L_homogenized',style='readonly',label="Effective stiffness\n(Mandel notation)"),
                      Item('kit',style='readonly',label='No. of iterations (kit)'),
                      Item('resnorm',style='readonly',label="Residual norm (resnorm)"),
                      label="Outputs",show_border=True),
                    label="FFTH Postprocessing"),
            title="Homogenization",height=0.6,width=0.4)    
        
    
#    view = View(
#        Group(Group(Item('ind_file',label="Info file"),
#                      Item('ind_update',label="Update",show_label=False),
#                      label="Input data from nanoindentation",show_border=True),
#            Group(Item('ind_N',label="No. of discretization points",style='readonly'),
#                            Item('ind_Y',label="Size of PUC",style='readonly'),
#                            Item('ind_h',label="Discretization step"),
#                            Item('ind_draw_E',label="Draw E",show_label=False),
#                            Item('ind_edit_E',label="Edit E",show_label=False),
#                            Item('ind_draw_nu',label="Draw nu",show_label=False),
#                            Item('ind_edit_nu',label="Edit nu",show_label=False),
#                            label="Data from input file",show_border=True),
#              Group(Item('ind_input_name',label="Name of input file"),
#                    Item('ind_make_input',label="Make input file",show_label=False),
#                            label="Creation of input file",show_border=True,),
#                    label="Indentation"),
#        Group(Group(Item('ind2_info_file',label="Info file"),
#                      Item('ind2_data_file',label="Data file",),
#                      Item('ind2_nu',label="Poisson\'s ratio"),
#                      label="Input data from nanoindentation",show_border=True),
#                      Group(Item('ind_input',label="Make input file"),
#                            Item('ind_input_name',label="Name of input file"),
#                            Item('ind_calc',label="Calculate"),
#                            Item('ind_postpro',label="Postprocess"),
#                            Item('ind_make_input',label="Make input file",show_label=False),
#                            label="Creation of input file",show_border=True),
#                    label="Indentation 2"),
#        Group(Group(Item('calc_file',label="Input file"),
#                      Item('tol',label="Tolerance for convergence"),
#                      Item('maxit',label="Maximal no. of iterations"),
#                      label="Inputs for FFTH homogenization",show_border=True),
#            Group(Item('calc_output_file',label="calc_output_file"),
#                    Item('calc',label="Calculate",show_label=False),
#                      label="Calculation",show_border=True),
#                    label="FFT Homogenization"),
#        Group(Group(Item('postproc_file'),
#                    Item('postproc_update',label="update",show_label=False),
#              label="Postprocessing file",show_border=True),
#              Group(Item('L_homogenized',style='readonly',label="Effective stiffness\n(Mandel notation)"),
#                      Item('kit',style='readonly',label='No. of iterations (kit)'),
#                      Item('resnorm',style='readonly',label="Residual norm (resnorm)"),
#                      label="Outputs",show_border=True),
#                    label="FFTH Postprocessing"),
#            title="Homogenization",height=0.6,width=0.4)
    

panel = ControlPanel()        
panel.configure_traits()
      
      
